@@ -1,6 +1,6 @@
 #!/usr/bin/perl -Tw
 
-# $Id: Hash.pm 1518 2010-08-22 23:56:21Z ian $
+# $Id: Hash.pm,v 1.3 2007-06-04 08:21:17 ian Exp $
 package Class::Declare::Hash;
 
 use strict;
@@ -20,8 +20,8 @@ L<Class::Declare>, providing the B<hash()> routine.
 use base  qw( Class::Declare     );
 use vars  qw( $REVISION $VERSION );
 
-  $REVISION = '$Revision: 1518 $';
-  $VERSION  = '0.19.1';	# Class::Declare->VERSION;
+  $REVISION = '$Revision: 1.3 $';
+  $VERSION  = '0.08';
 
 
 =head1 DESCRIPTION
@@ -52,7 +52,7 @@ L<Class::Declare>-derived object or package.
   # give access to the %__ATTR__, %__FRIEND__ and %__DEFN__ hashes of
   # Class::Declare. See the comment in Class::Declare::hash() for an
   # explanation.
-  sub __init__ : method
+  sub __init__ : locked method
   {
     my  $class  = __PACKAGE__->class( shift );
     # what's our method name?
@@ -64,19 +64,30 @@ L<Class::Declare>-derived object or package.
     $__GET_VALUES__     = $_[ 1 ] unless ( defined $__GET_VALUES__     );
     $__GET_FRIENDS__    = $_[ 2 ] unless ( defined $__GET_FRIENDS__    );
 
+    # about to do something that perhaps we shouldn't :)
+    {
+      no strict 'refs';
+
+      # remove this subroutine's entry from the symbol table
+      # that it can't be called again
+      undef *{ $sub };
+
+      # set the symbol table entry for Class::Declare::hash() to be
+      # the same as Class::Declare::Hash::hash()
+            $sub      = ( caller 2 )[ 3 ];
+      my  ( $method ) = ( $sub =~ m/([^:]+)$/o );
+
+         *{ $sub    } = *{ __PACKAGE__ . '::' . $method };
+    }
+
     1;  # that's all: hack complete :)
   }
 
 
-  # %__REFERENCES__
-  #
-  # Store attribute references for showing equality in the hash.
-  my  %__REFERENCES__;  undef %__REFERENCES__;
-
   # %__CALLER__
   #
   # Store the caller information for the original call to hash()
-  my  %__CALLER__;      undef %__CALLER__;
+  my  %__CALLER__;    undef %__CALLER__;
 
 
   #
@@ -106,8 +117,7 @@ L<Class::Declare>-derived object or package.
 
       # first, we must be certain that the target is derived from
       # Class::Declare
-      return undef      unless ( UNIVERSAL::isa( $target          ,
-                                                 'Class::Declare' ) );
+      return undef      unless ( $target->isa( 'Class::Declare' ) );
 
       # if we're testing class or abstract attributes, then that's all we need
       return 1              if ( $type eq 'class'    );
@@ -127,8 +137,8 @@ L<Class::Declare>-derived object or package.
       # not a friend, then we can't proceed
       my  $caller = $__CALLER__{ package    };
       my  $sub    = $__CALLER__{ subroutine };
-      return undef  unless (    UNIVERSAL::isa( $caller , $class  )
-                             || UNIVERSAL::isa( $class  , $caller )
+      return undef  unless (    $caller->isa( $class  )
+                             ||  $class->isa( $caller )
                              || $caller && exists $friend->{ $caller }
                              || $sub    && exists $friend->{ $sub    }
                            );
@@ -167,9 +177,6 @@ L<Class::Declare>-derived object or package.
       # calling routine/context
       $__CALLER__{ package    } = ( caller 1 )[ 0 ];
       $__CALLER__{ subroutine } = ( caller 2 )[ 3 ];
-
-      # reset the references store
-      undef %__REFERENCES__;
     }; # $__save__()
 
 
@@ -179,80 +186,13 @@ L<Class::Declare>-derived object or package.
   my  $__clear__  = sub {
       # clear the caller stack
       %__CALLER__ = ();
-
-      # reset the references store
-      undef %__REFERENCES__;
     }; # $__clear__()
 
 
-  # $__hash__()
-  #
-  # Perform a recursive hash() expansion for a given value
-  my  $__hash__;
-      $__hash__   = sub { # <r> , <depth> , <args>
-      my  $r          = shift;
-      my  $depth      = shift;
-
-      # if depth is zero, then return the value we have
-      return $r       unless ( ! defined $depth || $depth > 0 );
-
-      # if the value is undefined, then return undefined
-      return undef    unless (   defined $r );
-
-      # if we don't have a reference, then return the supplied value
-      return $r       unless (       ref $r );
-
-      # reduce the depth (if defined)
-          $depth--        if (   defined $depth );
-
-      # we have a reference value
-      #   - if it's an object derived from Class::Declare, then we should
-      #     call its hash() method and perform a recursive expansion
-      #   - if it's an ARRAY or HASH, we should iterate through its values
-      #     and attempt to expand them (if possible)
-      foreach ( ref $r ) {
-        # array
-        /^ARRAY$/o  && do {
-          my  $ref  = [];
-          push @{ $ref } , scalar $__hash__->( $_ , $depth , @_ )
-                                                        foreach ( @{ $r } );
-
-          # return the generated array
-          return $ref;
-        };
-
-        # hash
-        /^HASH$/o   && do {
-          my  $ref          = {};
-          while ( my ( $k , $v ) = each %{ $r } ) {
-              $ref->{ $k }  = $__hash__->( $v , $depth , @_ )
-          }
-
-          # return the generated hash
-          return $ref;
-        };
-
-        # are we dealing with a Class::Declare object that supports the hash()
-        # method?
-        #   - if so, recurse through that
-            UNIVERSAL::isa( $r , 'Class::Declare' )
-        and UNIVERSAL::can( $r , 'hash'           )
-        and return scalar $r->hash( @_ , depth => $depth );
-      }
-
-      # if we've made it this far, then simply return the value passed in
-      return $r;
-    };  # $__hash__()
-
-
-# jump into the Class::Declare namespace to create the dump() routine
-package Class::Declare;
-
-
 # hash()
 #
 # Generate a textual representation of the object/class
-sub hash : method
+sub hash : locked method
 {
   my  $self   = Class::Declare->class( shift );
   my  $class  = ref( $self ) || $self;
@@ -265,8 +205,6 @@ sub hash : method
                                            static     => undef ,
                                            restricted => undef ,
                                            abstract   => undef ,
-                                           depth      => undef ,
-                                           backtrace  => 1     ,
                                            all        => 1     } );
 
   # have we been called from outside this file
@@ -274,13 +212,10 @@ sub hash : method
   my  $outside  = ( caller )[ 1 ] ne __FILE__;
 
   # if we're called from outside this file (i.e. it's not an
-  # internal recursive call to hash()) then make
+  # internal recursive call to hash() from $__dump__()) then make
   # note of the arguments and the context
     $__save__->( $self , $_args ) if ( $outside );
 
-  # store the current depth limit
-  my  $depth    = delete $_args->{ depth };
-
   # unset 'all' if any of the other arguments have been set
   ( $_args->{ $_ } )
     and delete $_args->{ all }
@@ -315,9 +250,7 @@ sub hash : method
         die "access to $_ attributes denied in call to "
             . "$sub() at $file line $line\n";
       } foreach ( grep { $_args->{ $_ } }
-                       grep {    !/all/o
-                              && !/backtrace/o
-                            } keys %{ $_args } );
+                       grep { !/all/ } keys %{ $_args } );
   }
 
   # determine the attribute types that may be returned/have been requested
@@ -394,44 +327,7 @@ sub hash : method
                                            grep { defined }
                                                 $rmap{ $type } ) {
 
-      # what value do we have?
-      my  $v            = $hash->{ $attr };
-
-      # if this is a reference
-      if ( ref $v ) {
-        # if we have backtrace turned on, then check to see if we have
-        # seen this reference before
-        my  $r          = $__REFERENCES__{ $v };
-
-        # if we've not seen this reference before, then we should attempt
-        # to expand it
-        unless ( defined $r ) {
-          # if we have not reached our depth limit, then recurse if we need to
-          #   - if the depth has not been given, then we descend as far
-          #     as we can
-          #   - NOTE: this is a change in default behaviour since v0.08
-          if ( ! defined $depth || $depth > 0 ) {
-            # generate the expansion of this value
-            #   - decrement the depth count
-            #$depth--      if ( defined $depth );
-            $r          = $__hash__->( $v , $depth , %{ $_args } );
-          }
-
-          # if we don't have a reference, then use the original value
-            $r        ||= $v;
-
-          # the value we have now is all we are going to get for this
-          # attribute, so make sure it's stored (if we have backtracing turned
-          # on)
-            $__REFERENCES__{ $v } =  $r     if ( $_args->{ backtrace } );
-        }
-
-        # use whatever expansion we have obtained
-          $v            = $r;
-      }
-
-      # record the expansion for this attribute
-          $rtn{ $attr } = $v;
+          $rtn{ $attr } = $hash->{ $attr };
     }
   }
 
@@ -453,11 +349,11 @@ L<Class::Declare>
 
 =head1 AUTHOR
 
-Ian Brayshaw, E<lt>ibb@cpan.orgE<gt>
+Ian Brayshaw, E<lt>ian@onemore.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003-2016 by Ian Brayshaw. All rights reserved.
+Copyright 2003-2007 by Ian Brayshaw. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
