@@ -1,14 +1,17 @@
 #!/usr/bin/perl -w
-# $Id: Test.pm,v 1.19 2003/06/06 10:42:42 ian Exp $
+# $Id: Test.pm,v 1.21 2007-03-04 23:30:37 ian Exp $
 
 # generate and execute access permutations
 package Class::Declare::Test;
 
 use strict;
+use Class::Declare;
+use File::Spec::Functions;
+
 use base qw( Class::Declare     );
 use vars qw( $REVISION $VERSION );
-             $REVISION	= '$Revision: 1.19 $';
-			 $VERSION	= '0.01';
+             $REVISION	= '$Revision: 1.21 $';
+             $VERSION   = '0.02';
 
 =head1 NAME
 
@@ -29,9 +32,9 @@ Class::Declare::Test - simplify the generation of method/attribute tests.
                 ... ];
 
   # create the test object
-  my $test  = Class::Declare::Test->new( type   => $type  ,
-                                         tests  => $tests ,
-                                         strict => 1      );
+  my $test  = Class::Declare::Test->new( type  => $type  ,
+                                         tests => $tests ,
+                                         check => 1      );
 
   # run the tests
      $test->run;
@@ -42,96 +45,101 @@ Class::Declare::Test - simplify the generation of method/attribute tests.
 # define the testing constants
 #
 
-use constant	_CLASS				=> ( 1 << 0 );
-use constant	_DERIVED			=> ( 1 << 1 );
-use constant	_UNRELATED			=> ( 1 << 2 );
+use constant	_CLASS				      => ( 1 << 0 );
+use constant	_DERIVED			      => ( 1 << 1 );
+use constant	_UNRELATED		    	=> ( 1 << 2 );
+use constant	_PARENT   		    	=> ( 1 << 3 );
 
 # are we dealing with a class or instance?
-use constant	_INSTANCE			=> ( 1 << 3 );
-use constant	_IS_INSTANCE		=> sub {   $_[ 0 ] >> 3 };
+use constant	_INSTANCE			      => ( 1 << 4 );
+use constant	_IS_INSTANCE	    	=> sub {   $_[ 0 ] >> 4 };
 
 # define the context and target macros
-use constant	_CONTEXT			=> sub {   $_[ 0 ] << 0 };
-use constant	_TARGET				=> sub {   $_[ 0 ] << 4 };
+use constant	_CONTEXT			      => sub {   $_[ 0 ] << 0 };
+use constant	_TARGET			      	=> sub {   $_[ 0 ] << 5 };
 
 # extract the context, context and target from a given bitmap
-use constant	_MASK				=> ( ( 1 << 4 ) - 1 );
-use constant	_GET_CONTEXT		=> sub { ( $_[ 0 ] >> 0 ) & _MASK };
-use constant	_GET_TARGET			=> sub { ( $_[ 0 ] >> 4 ) & _MASK };
+use constant	_MASK				        => ( ( 1 << 5 ) - 1 );
+use constant	_GET_CONTEXT		    => sub { ( $_[ 0 ] >> 0 ) & _MASK };
+use constant	_GET_TARGET			    => sub { ( $_[ 0 ] >> 5 ) & _MASK };
 
 # define the test constants
-use constant	_TEST_ACCESS		=> ( 1 << 0 );
-use constant	_TEST_READ			=> ( 1 << 1 );
+use constant	_TEST_ACCESS		    => ( 1 << 0 );
+use constant	_TEST_READ			    => ( 1 << 1 );
 use constant	_TEST_WRITE_LVALUE	=> ( 1 << 2 );
-use constant	_TEST_WRITE_ARG		=> ( 1 << 3 );
-use constant	_TEST_WRITE			=> (   _TEST_WRITE_LVALUE
-            	            		     | _TEST_WRITE_ARG    );
-use constant	_TEST_ALL			=> (   _TEST_ACCESS
-            	         			     | _TEST_READ
-            	         			     | _TEST_WRITE        );
+use constant	_TEST_WRITE_ARG		  => ( 1 << 3 );
+use constant	_TEST_WRITE			    => (   _TEST_WRITE_LVALUE
+            	            		         | _TEST_WRITE_ARG    );
+use constant	_TEST_ALL			      => (   _TEST_ACCESS
+            	         			           | _TEST_READ
+            	         			           | _TEST_WRITE        );
 
-use constant	_TEST				=> sub {   $_[ 0 ] << 8 };
-use constant	_GET_TEST			=> sub { ( $_[ 0 ] >> 8 ) & _MASK };
+use constant	_TEST				        => sub {   $_[ 0 ] << 10 };
+use constant	_GET_TEST		       	=> sub { ( $_[ 0 ] >> 10 ) & _MASK };
 
 # are we dealing in the method call or the attribute
-use constant	_ATTRIBUTE			=> ( 1 << 0 );
-use constant	_METHOD				=> 0;
+use constant	_ATTRIBUTE	    		=> ( 1 << 0 );
+use constant	_METHOD			      	=> 0;
 
 # define the constants for the focus of the test
-use constant	_FOCUS				=> sub {   $_[ 0 ] << 12 };
-use constant	_GET_FOCUS			=> sub { ( $_[ 0 ] >> 12 ) & 1 };
+use constant	_FOCUS			      	=> sub {   $_[ 0 ] << 14 };
+use constant	_GET_FOCUS	    		=> sub { ( $_[ 0 ] >> 14 ) & 1 };
 
 # define the expected result of the test
-use constant	_DIE 				=>   0;
-use constant	_LIVE				=> ( 1 << 0 );
-use constant	_RESULT				=> sub {   $_[ 0 ] << 13 };
-use constant	_GET_RESULT			=> sub { ( $_[ 0 ] >> 13 ) & 1 };
+use constant	_DIE 				        =>   0;
+use constant	_LIVE				        => ( 1 << 0 );
+use constant	_RESULT			      	=> sub {   $_[ 0 ] << 15 };
+use constant	_GET_RESULT	    		=> sub { ( $_[ 0 ] >> 15 ) & 1 };
 
 # define the context macros to be exported
-use constant	CTX_CLASS			=> _CONTEXT->(             _CLASS     );
-use constant	CTX_DERIVED			=> _CONTEXT->(             _DERIVED   );
-use constant	CTX_UNRELATED		=> _CONTEXT->(             _UNRELATED );
-use constant	CTX_INSTANCE		=> _CONTEXT->( _INSTANCE | _CLASS     );
-use constant	CTX_INHERITED		=> _CONTEXT->( _INSTANCE | _DERIVED   );
-use constant	CTX_FOREIGN			=> _CONTEXT->( _INSTANCE | _UNRELATED );
+use constant	CTX_CLASS		   	    => _CONTEXT->(             _CLASS     );
+use constant	CTX_DERIVED			    => _CONTEXT->(             _DERIVED   );
+use constant	CTX_UNRELATED		    => _CONTEXT->(             _UNRELATED );
+use constant	CTX_PARENT   		    => _CONTEXT->(             _PARENT    );
+use constant	CTX_INSTANCE		    => _CONTEXT->( _INSTANCE | _CLASS     );
+use constant	CTX_INHERITED		    => _CONTEXT->( _INSTANCE | _DERIVED   );
+use constant	CTX_FOREIGN			    => _CONTEXT->( _INSTANCE | _UNRELATED );
+use constant	CTX_SUPER  			    => _CONTEXT->( _INSTANCE | _PARENT    );
 
 # define the target macros to be exported
-use constant	TGT_CLASS			=>  _TARGET->(             _CLASS     );
-use constant	TGT_DERIVED			=>  _TARGET->(             _DERIVED   );
-use constant	TGT_UNRELATED		=>  _TARGET->(             _UNRELATED );
-use constant	TGT_INSTANCE		=>  _TARGET->( _INSTANCE | _CLASS     );
-use constant	TGT_INHERITED		=>  _TARGET->( _INSTANCE | _DERIVED   );
-use constant	TGT_FOREIGN			=>  _TARGET->( _INSTANCE | _UNRELATED );
+use constant	TGT_CLASS		  	    =>  _TARGET->(             _CLASS     );
+use constant	TGT_DERIVED			    =>  _TARGET->(             _DERIVED   );
+use constant	TGT_UNRELATED		    =>  _TARGET->(             _UNRELATED );
+use constant	TGT_PARENT   		    =>  _TARGET->(             _PARENT    );
+use constant	TGT_INSTANCE		    =>  _TARGET->( _INSTANCE | _CLASS     );
+use constant	TGT_INHERITED		    =>  _TARGET->( _INSTANCE | _DERIVED   );
+use constant	TGT_FOREIGN			    =>  _TARGET->( _INSTANCE | _UNRELATED );
+use constant	TGT_SUPER  			    =>  _TARGET->( _INSTANCE | _PARENT    );
 
 # define the test macros to be exported
-use constant	TST_ACCESS			=>    _TEST->( _TEST_ACCESS           );
-use constant	TST_READ			=>    _TEST->( _TEST_READ             );
-use constant	TST_WRITE_LVALUE	=>    _TEST->( _TEST_WRITE_LVALUE     );
-use constant	TST_WRITE_ARG		=>    _TEST->( _TEST_WRITE_ARG        );
-use constant	TST_WRITE			=>    _TEST->( _TEST_WRITE            );
-use constant	TST_ALL				=>    _TEST->( _TEST_ALL              );
+use constant	TST_ACCESS	  		  =>    _TEST->( _TEST_ACCESS           );
+use constant	TST_READ		    	  =>    _TEST->( _TEST_READ             );
+use constant	TST_WRITE_LVALUE	  =>    _TEST->( _TEST_WRITE_LVALUE     );
+use constant	TST_WRITE_ARG 		  =>    _TEST->( _TEST_WRITE_ARG        );
+use constant	TST_WRITE		    	  =>    _TEST->( _TEST_WRITE            );
+use constant	TST_ALL			    	  =>    _TEST->( _TEST_ALL              );
 
 # define the result macros to be exported
-use constant	LIVE				=>  _RESULT->( _LIVE                  );
-use constant	DIE					=>  _RESULT->( _DIE                   );
+use constant	LIVE				        =>  _RESULT->( _LIVE                  );
+use constant	DIE					        =>  _RESULT->( _DIE                   );
 
 # define the focus macros for export
-use constant	ATTRIBUTE			=>   _FOCUS->( _ATTRIBUTE             );
-use constant	METHOD				=>   _FOCUS->( _METHOD                );
+use constant	ATTRIBUTE			      =>   _FOCUS->( _ATTRIBUTE             );
+use constant	METHOD				      =>   _FOCUS->( _METHOD                );
 
 
 # define the export targets
 use vars qw/ @EXPORT_OK %EXPORT_TAGS /;
 
 # define attribute and method default values
-use constant	DEFAULT_ATTRIBUTE	=> rand time;
-use constant	DEFAULT_METHOD		=> rand time;
+use constant	DEFAULT_ATTRIBUTE	  => rand time;
+use constant	DEFAULT_METHOD		  => rand time;
 
 
 # define the accessors we'll use in the test classes
 #    - these accessors ensure we have the correct context for all of
 #      the test cases
-use constant	ACCESSORS			=> <<__EODfN__;
+use constant	ACCESSORS			      => <<__EODfN__;
 
 #
 # We need to test to see whether we can access attributes and methods from
@@ -146,7 +154,7 @@ sub get
 	# will be honoured as either a class or instance method
 	my	\$self		= __PACKAGE__->class( shift );
 	my	\$target	= shift || \$self;
-		\$target->attribute
+  		\$target->attribute
 } # get()
 
 
@@ -156,7 +164,7 @@ sub call
 	# will be honoured as either a class or instance method
 	my	\$self		= __PACKAGE__->class( shift );
 	my	\$target	= shift || \$self;
-		\$target->method;
+  		\$target->method;
 } # call()
 
 
@@ -171,8 +179,8 @@ sub cmp_lvalue
 	my	\$self		= __PACKAGE__->class( shift );
 	my	\$target	= shift || \$self;
 	my	\$rand		= rand time;
-		\$target->lvalue( \$rand );
-	( \$target->attribute == \$rand );
+  		\$target->lvalue( \$rand );
+  	( \$target->attribute == \$rand );
 } # cmp_lvalue()
 
 
@@ -182,8 +190,8 @@ sub cmp_argument
 	my	\$self		= __PACKAGE__->class( shift );
 	my	\$target	= shift || \$self;
 	my	\$rand		= rand time;
-		\$target->argument( \$rand );
-	( \$target->attribute == \$rand );
+  		\$target->argument( \$rand );
+  	( \$target->attribute == \$rand );
 } # cmp_argument()
 
 __EODfN__
@@ -346,7 +354,7 @@ I<result>. See the C<class.t>, C<public.t>, etc test scripts for examples.
 	my	@symbols	= keys %{ __PACKAGE__ . '::' };
 	my	@context	= grep { /^CTX_/o } @symbols;
 	my	@target		= grep { /^TGT_/o } @symbols;
-	my	@test		= grep { /^TST_/o } @symbols;
+	my	@test		  = grep { /^TST_/o } @symbols;
 	my	@focus		= qw( ATTRIBUTE METHOD );
 	my	@result		= qw( LIVE      DIE    );
 
@@ -372,7 +380,7 @@ use Test::Exception;
 =over 4
 
 =item B<new(> type => I<type> , tests => I<tests>
-              [ , strict => I<boolean> ] B<)>
+              [ , check => I<boolean> ] B<)>
 
 Create a new test object. I<type> specifies the type of attribute/method to
 test, which must be one of the following:
@@ -397,8 +405,8 @@ I<tests> is a reference to an array of test bitmaps defining the tests
 to perform. See the C<class>, C<static>, C<restricted>, C<public>, C<private>,
 C<protected> and C<strict> test files.
 
-The I<strict> attribute may be used to turn strict access checking on and
-off for a particular set of tests. I<strict> defaults to true, giving strict
+The I<check> attribute may be used to turn strict access checking on and
+off for a particular set of tests. I<check> defaults to true, giving strict
 access checking, while a false value will turn access checking off.
 
 =cut
@@ -407,22 +415,23 @@ access checking, while a false value will turn access checking off.
 __PACKAGE__->declare(
 
 	# public attributes
-	public  => { type      => undef ,	# type of test (public, private, etc)
-	             tests     => undef ,	# the tests hash
-	             strict    => undef } ,	# strict access checking (inherited
-	       	                           	#   from Class::Declare)
+	public  => { type      => undef ,	  # type of test (public, private, etc)
+	             tests     => undef ,	  # the tests hash
+	             check     => undef } ,	# turn on strict checking
 
 	# private attributes for the Permute class
-	private => { base      => undef ,	# the base class
-	             derived   => undef ,	# the derived class
-	             unrelated => undef ,	# the unrelated class
-	             instance  => undef ,	# the base class instance
-	             inherited => undef ,	# the derived class instance
-	             foreign   => undef } ,	# the unrelated class instance
+	private => { base      => undef ,	  # the base class
+	             derived   => undef ,	  # the derived class
+	             unrelated => undef ,	  # the unrelated class
+	             parent    => undef ,	  # the parent class
+	             instance  => undef ,	  # the base class instance
+	             inherited => undef ,	  # the derived class instance
+	             foreign   => undef ,	  # the unrelated class instance
+	             super     => undef } ,	# the parent class instance
 
 	# specify the initialisation routine
 	init    => sub {
-		my	$self	= __PACKAGE__->public( shift );
+		my	$self	  = __PACKAGE__->public( shift );
 		my	$class	= ref( $self );
 
 		# ensure the test type and outcomes hash have been define
@@ -433,7 +442,8 @@ __PACKAGE__->declare(
 
 		# make sure the type is understood
 		( grep { $self->type eq $_ } qw( class  static  restricted 
-		                                 public private protected  ) )
+		                                 public private protected  
+                                                    abstract   ) )
 			 or warn $class . ': unknown type "' . $self->type . '"'
 			and return undef;
 
@@ -443,35 +453,65 @@ __PACKAGE__->declare(
 			and return undef;
 
 		# create the base, derived and unrelated class names
-		my	$type				= $self->type;
-			$self->base			= join '::' , __PACKAGE__ , ucfirst( $type );
-			$self->derived		= join '::' , $self->base , 'Derived';
-			$self->unrelated	= join '::' , $self->base , 'Unrelated';
+		my	$type				        = $self->type;
+			  $self->base			    = join '::' , __PACKAGE__ , ucfirst( $type );
+			  $self->derived		  = join '::' , $self->base , 'Derived';
+			  $self->unrelated	  = join '::' , $self->base , 'Unrelated';
+			  $self->parent       = join '::' , $self->base , 'Parent';
 
 		# make note of the default method and attribute values
-		my	$default_method		= DEFAULT_METHOD;
+		my	$default_method		  = DEFAULT_METHOD;
 		my	$default_attribute	= DEFAULT_ATTRIBUTE;
 
 		# make a copy of the accessors' code
-		my	$accessors			= ACCESSORS;
+		my	$accessors			    = ACCESSORS;
 
 		# do we have access checking?
-		my	$strict				=  ( defined $self->strict )
-		  	       				           ? $self->strict : 'undef';
+		my	$strict				      =  ( defined $self->check )
+		  	       				                 ? $self->check : 'undef';
+
+		# create the parent class
+    #   - provided it hasn't been created before
+    my  $pkg    = $self->parent;
+    #   - convert the package name into a file name
+    my  $file   = catfile( split '::' , $pkg ) . '.pm';
+		unless ( $INC{ $file } ) {
+			my	$dfn	= <<__EODfN__;
+package $pkg;
+
+use strict;
+use base qw( Class::Declare );
+
+# add the accessors
+$accessors
+
+1;
+__EODfN__
+
+		  # attempt to instatiate this package
+	    eval $dfn
+		    or warn __PACKAGE__ , ": failed to create $pkg:\n\t$@"
+		   and return undef;
+    }
 
 		# define the packages
 		#   NB: only define base and unrelated here, the derived
 		#       class simply inherits everything from base
 		foreach my $pkg ( map { $self->$_() } qw( base unrelated ) ) {
 			# if this package has already been defined then ignore it
-			next		if ( defined $INC{ $pkg } );
+      #   - convert the package name into a file name
+      my  $file   = catfile( split '::' , $pkg ) . '.pm';
+			next		    if ( defined $INC{ $file } );
+
+      # does this class have a parent class?
+      my  $parent = ( $pkg eq $self->base ) ? $self->parent : '';
 
 			# create the package definition
-			my	$dfn	= <<__EODfN__;
+			my	$dfn	  = <<__EODfN__;
 package $pkg;
 
 use strict;
-use base qw( Class::Declare );
+use base qw( Class::Declare $parent );
 
 # define the $type attribute
 #    - do we want strict access checking?
@@ -506,7 +546,7 @@ $accessors
 sub lvalue
 {
 	my	\$self	= __PACKAGE__->public( shift );
-		\$self->attribute	= shift;
+		  \$self->attribute	= shift;
 } # lvalue()
 
 
@@ -514,7 +554,7 @@ sub lvalue
 sub argument
 {
 	my	\$self	= __PACKAGE__->public( shift );
-		\$self->attribute( shift );
+		  \$self->attribute( shift );
 } # argument()
 
 
@@ -529,7 +569,7 @@ sub reset
 	my	\$self	= __PACKAGE__->public( shift );
 	# NB: use argument style so that non-modifiable attributes will silently
 	#     fail (other parts of the tests should pick this up)
-		\$self->attribute( shift );
+		  \$self->attribute( shift );
 } # reset()
 
 
@@ -542,11 +582,14 @@ __EODfN__
 					and return undef;
 		}
 
-		# create the derived class (if it hasn't been created before)
-		my	$pkg	= $self->derived;
-		unless ( $INC{ $pkg } ) {
-			my	$base	= $self->base;
-			my	$dfn	= <<__EODfN__;
+		# create the derived class
+    #   - provided it hasn't been created before
+        $pkg  = $self->derived;
+    #   - convert the package name into a file name
+        $file   = catfile( split '::' , $pkg ) . '.pm';
+		unless ( $INC{ $file } ) {
+		  my	$base	= $self->base;
+      my	$dfn	= <<__EODfN__;
 package $pkg;
 
 use strict;
@@ -558,16 +601,17 @@ $accessors
 1;
 __EODfN__
 
-		# attempt to instatiate this package
-			eval $dfn
-				or warn __PACKAGE__ , ": failed to create $pkg:\n\t$@"
-					and return undef;
-		}
+		  # attempt to instatiate this package
+      eval $dfn
+        or warn __PACKAGE__ , ": failed to create $pkg:\n\t$@"
+       and return undef;
+    }
 
 		# create the object instances
-		$self->instance		= $self->base->new			or return undef;
-		$self->inherited	= $self->derived->new		or return undef;
-		$self->foreign		= $self->unrelated->new		or return undef;
+		$self->instance		= $self->base->new			  or return undef;
+		$self->inherited	= $self->derived->new		  or return undef;
+		$self->foreign		= $self->unrelated->new   or return undef;
+		$self->super  		= $self->parent->new      or return undef;
 
 		1;	# everything is OK
 	} # init()
@@ -577,47 +621,72 @@ __EODfN__
 
 { # closure for extracting the required context & target
 
-	# $get()
+  # define the mapping between bitmaps and names
+  my  @__NAME__   = ();
+      @__NAME__[ map { (   $_               ,
+                         ( $_ | _INSTANCE ) )
+                     } ( _CLASS , _DERIVED , _UNRELATED , _PARENT )
+               ]  = qw(   base      instance derived inherited
+                          unrelated foreign  parent  super        );
+      
+  # $name()
+  #
+  # Extract the name of the method specified by the given bitmap
+  my  $name = sub { return $__NAME__[ $_[ 0 ] ] }; # $name()
+
+
+	# $code()
 	#
 	# Extract the required instance/class
-	my	$get	= sub {
-			# are we dealing with an instance...
-			if ( _IS_INSTANCE->( $_[ 1 ] ) ) {
-				return $_[ 0 ]->instance	if ( $_[ 1 ] & _CLASS     );
-				return $_[ 0 ]->inherited	if ( $_[ 1 ] & _DERIVED   );
-				return $_[ 0 ]->foreign		if ( $_[ 1 ] & _UNRELATED );
+	my	$code	= sub {
+                my  $method = $name->( $_[ 1 ] );
 
-			# ... or a class?
-			} else {
-				return $_[ 0 ]->base		if ( $_[ 1 ] & _CLASS     );
-				return $_[ 0 ]->derived		if ( $_[ 1 ] & _DERIVED   );
-				return $_[ 0 ]->unrelated	if ( $_[ 1 ] & _UNRELATED );
-			}
-			
-			return undef;	# couldn't determine the instance/class
-		}; # $get()
+                return $_[ 0 ]->$method();
+              };  # $code()
 
 # context()
 #
 # Extract the context from the given test code.
 sub context
 {
-	my	$self	= __PACKAGE__->private( shift );
+	my	$self	= __PACKAGE__->private( $_[ 0 ] );
 
-	return $get->( $self , _GET_CONTEXT->( $_[ 0 ] ) );
+	return $code->( $self , _GET_CONTEXT->( $_[ 1 ] ) );
 } # context()
 
 
+# context_string()
+#
+# Extract a string representation for the caller enviornment.
+sub context_string
+{
+  my  $self = __PACKAGE__->private( $_[ 0 ] );
+
+  return $name->( _GET_CONTEXT->( $_[ 1 ] ) );
+} # context_string()
+
+
 # target()
 #
 # Extract the target from the given test code.
 sub target
 {
-	my	$self	= __PACKAGE__->private( shift );
+	my	$self	= __PACKAGE__->private( $_[ 0 ] );
 
-	return $get->( $self , _GET_TARGET->( $_[ 0 ] ) );
+	return $code->( $self , _GET_TARGET->( $_[ 1 ] ) );
 } # target()
 
+
+# target_string()
+#
+# Extract a string representation for the caller enviornment.
+sub target_string
+{
+  my  $self = __PACKAGE__->private( $_[ 0 ] );
+
+  return $name->( _GET_TARGET->( $_[ 1 ] ) );
+} # target_string()
+
 } # end of context/target closure
 
 
@@ -626,8 +695,8 @@ sub target
 # Extract the focus from the given test code.
 sub focus
 {
-	my	$self	= __PACKAGE__->private( shift );
-	my	$focus	= _GET_FOCUS->( $_[ 0 ] );
+	my	$self	  = __PACKAGE__->private( $_[ 0 ] );
+	my	$focus	= _GET_FOCUS->( $_[ 1 ] );
 
 	return 'attribute'		if ( $focus & _ATTRIBUTE );
 	return 'method';
@@ -681,6 +750,10 @@ sub run
 {
 	my	$self	= __PACKAGE__->public( shift );
 
+  # generate the test message
+  my  $msg  = sub { '(from ' . $self->context_string( $_[ 0 ] ) . ' on '
+                             . $self->target_string(  $_[ 0 ] ) . ')'    };
+
 	# run through each test
 	TYPE: foreach my $type ( map { @{ $_ } } $self->tests ) {
 		# determine the context and target
@@ -688,9 +761,9 @@ sub run
 		my	$target		= $self->target( $type );
 
 		# what tests are we to perform?
-		my	$test		= $self->test  ( $type );
+		my	$test		  = $self->test  ( $type );
 		# do we want this test to live or die?
-		my	$live		= $self->result( $type );
+		my	$live		  = $self->result( $type );
 
 		# now we need to determine the focus of this test
 		# i.e. are we interested in an attribute or a method?
@@ -707,26 +780,26 @@ sub run
 				( $test & _TEST_ACCESS )	&& do {
 					if ( $live ) {
 						lives_ok { $context->get( $target ) }
-					         	'attribute access honoured';
+					         	'attribute access honoured ' . $msg->( $type );
 					} else {
 						 dies_ok { $context->get( $target ) }
-						        'attribute access forbidden';
+						        'attribute access forbidden ' . $msg->( $type );
 					}
 				};
 
 				# reset the attribute values
-					$self->reset;
+				$self->reset      unless ( $self->type eq 'abstract' );
 
 				# need to check to see if we can access the attribute
 				( $test & _TEST_READ   )	&& do {
 					if ( $live ) {
 						lives_and {
 							is $context->get( $target ) , DEFAULT_ATTRIBUTE
-						} 'attribute read honoured';
+						} 'attribute read honoured ' . $msg->( $type );
 					} else {
 						  dies_ok {
 						 	   $context->get( $target )
-						} 'attribute read forbidden';
+						} 'attribute read forbidden ' . $msg->( $type );
 					}
 				};
 
@@ -735,26 +808,26 @@ sub run
 				( $test & _TEST_WRITE_ARG )	&& do {
 					if ( $live ) {
 						ok( $context->cmp_argument( $target )     ,
-						    'attribute write argument honoured' );
+						    'attribute write argument honoured ' . $msg->( $type ) );
 					} else {
 						dies_ok { $context->cmp_argument( $target ) or die }
-						    'attribute write argument forbidden';
+						    'attribute write argument forbidden ' . $msg->( $type );
 					}
 				};
 
 				# reset the attribute values
-					$self->reset;
+				  $self->reset      unless ( $self->type eq 'abstract' );
 
 				# need to check with writing to an attribute as
 				# lvalue
 				( $test & _TEST_WRITE_LVALUE )	&& do {
 					if ( $live ) {
 						ok( $context->cmp_lvalue( $target )     ,
-						    'attribute write lvalue honoured' );
+						    'attribute write lvalue honoured ' . $msg->( $type ) );
 					} else {
 						dies_ok {
 							$context->cmp_lvalue( $target )
-						} 'attribute write lvalue forbidden';
+						} 'attribute write lvalue forbidden ' . $msg->( $type );
 					}
 				};
 			};
@@ -770,10 +843,10 @@ sub run
 				( $test & _TEST_ACCESS )	&& do {
 					if ( $live ) {
 						lives_ok { $context->call( $target ) }
-						         'method access honoured';
+						         'method access honoured ' . $msg->( $type );
 					} else {
 						 dies_ok { $context->call( $target ) }
-						         'method access forbidden';
+						         'method access forbidden ' . $msg->( $type );
 					}
 				};
 
@@ -781,12 +854,12 @@ sub run
 				( $test & _TEST_READ )		&& do {
 					if ( $live ) {
 						lives_and {
-							is $context->call( $target ) , DEFAULT_METHOD
-						} 'method read honoured';
+							  is $context->call( $target ) , DEFAULT_METHOD
+						} 'method read honoured ' . $msg->( $type );
 					} else {
 						  dies_ok {
 						  	is $context->call( $target ) , DEFAULT_METHOD
-						} 'method read forbidden';
+						} 'method read forbidden '. $msg->( $type );
 					}
 				};
 			};
